import { initialize as initializeWorker } from 'monaco-worker-manager/worker';
import { VFile } from 'vfile';
/**
 * Represent a vfile message in Monaco editor.
 *
 * @param message - The vfile message to represent in Monaco editor
 * @returns The vfile message as Monaco editor marker data.
 */
function vfileMessageToMarkerData(message) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    const msg = [];
    if (message.reason) {
        msg.push(message.reason);
    }
    if (message.note) {
        msg.push(message.note);
    }
    if (message.stack) {
        msg.push(message.stack);
    }
    return {
        severity: message.fatal == null ? 1 : message.fatal ? 8 : 4,
        startLineNumber: (_c = (_b = (_a = message.position) === null || _a === void 0 ? void 0 : _a.start.line) !== null && _b !== void 0 ? _b : message.line) !== null && _c !== void 0 ? _c : 0,
        startColumn: (_f = (_e = (_d = message.position) === null || _d === void 0 ? void 0 : _d.start.column) !== null && _e !== void 0 ? _e : message.column) !== null && _f !== void 0 ? _f : 0,
        endLineNumber: (_j = (_h = (_g = message.position) === null || _g === void 0 ? void 0 : _g.end.line) !== null && _h !== void 0 ? _h : message.line) !== null && _j !== void 0 ? _j : 0,
        endColumn: (_m = (_l = (_k = message.position) === null || _k === void 0 ? void 0 : _k.end.column) !== null && _l !== void 0 ? _l : message.column) !== null && _m !== void 0 ? _m : 0,
        message: msg.join('\n\n'),
        code: (_o = message.ruleId) !== null && _o !== void 0 ? _o : undefined,
        source: (_p = message.source) !== null && _p !== void 0 ? _p : undefined,
        expected: (_q = message.expected) !== null && _q !== void 0 ? _q : undefined,
        url: (_r = message.url) !== null && _r !== void 0 ? _r : undefined,
    };
}
/**
 * Initialize the worker.
 *
 * @param getProcessor - A function for getting a processor.
 */
export function initialize(getProcessor) {
    initializeWorker((ctx, createData) => {
        const getVFile = (uri) => {
            const models = ctx.getMirrorModels();
            for (const model of models) {
                if (String(model.uri) === uri) {
                    return new VFile({
                        path: new URL(uri),
                        value: model.getValue(),
                    });
                }
            }
        };
        return {
            async doValidate(uri) {
                const file = getVFile(uri);
                if (!file) {
                    return;
                }
                const processor = getProcessor(file, createData);
                const { messages } = await processor.process(file);
                return messages.map(vfileMessageToMarkerData);
            },
            async doFormat(uri) {
                const file = getVFile(uri);
                if (!file) {
                    return;
                }
                const processor = getProcessor(file, createData);
                const { value } = await processor.process(file);
                return value;
            },
        };
    });
}
